class GeometricCMBPrior:
    def __init__(self, mean, covariance):
        """An easy-to-work-with summary of CMB measurements

        Parameters:
        -----------
        mean: best-fit values for Omega_ch2, Omega_bh2, and 100tetha_MC

        covariance: covariance matrix of vector mean
        """
        self.mean = jnp.array(mean)
        self.cov = jnp.array(covariance)
        self.W = jnp.linalg.inv(self.cov)
        self.L = jnp.linalg.cholesky(self.W)

    def model(self, params):
        Omega_c_h2 = Omega_c(params) * (params["H0"] ** 2 * 1e-4)
        zstar = z_star(params)
        rsstar = rs(params, zstar)
        thetastar = rsstar / dM(params, zstar) * 100.0
        return jnp.array([Omega_c_h2, params["Omega_b_h2"], thetastar])

    def residuals(self, params):
        return self.mean - self.model(params)

    def weighted_residuals(self, params):
        return self.L @ self.residuals(params)

    def likelihood(self, params):
        r = self.weighted_residuals(params)
        return r.T @ r


# Extracted from
planck2018_prior = GeometricCMBPrior(
    [2.2337930e-02, 1.2041740e-01, 1.0409010e00],
    [
        [2.2139987e-08, -1.1786703e-07, 1.6777190e-08],
        [-1.1786703e-07, 1.8664921e-06, -1.4772837e-07],
        [1.6777190e-08, -1.4772837e-07, 9.5788538e-08],
    ],
)
